/***********************************************************************/
/** 	© 2015 CD PROJEKT S.A. All rights reserved.
/** 	THE WITCHER® is a trademark of CD PROJEKT S. A.
/** 	The Witcher game is based on the prose of Andrzej Sapkowski.
/***********************************************************************/



class W3Potion_Blizzard extends CBaseGameplayEffect
{
	private saved var speedMultiplierID : int;		
	private var slowdownFactor : float;

	default effectType = EET_Blizzard;
	default attributeName = 'slow_motion';

	event OnEffectAdded(optional customParams : W3BuffCustomParams)
	{	
		super.OnEffectAdded(customParams);	
		
		slowdownFactor = CalculateAttributeValue(effectValue);

		InitiateSlowMo();
	}
	
	public final function IsSlowMoActive() : bool
	{
		return speedMultiplierID >= 0;
	}
	
	public function KilledEnemy()
	{
	}
	
	public function OnLoad(t : CActor, eff : W3EffectManager)
	{
		super.OnLoad(t, eff);
		RemoveSlowMo();
	}
	
	public function OnTimeUpdated(dt : float)
	{
		if(speedMultiplierID >= 0)
			super.OnTimeUpdated(dt / slowdownFactor);
		else
			super.OnTimeUpdated(dt);

		if (target.IsInCombat() && !IsSlowMoActive())
			InitiateSlowMo();
		else if (!target.IsInCombat() && IsSlowMoActive())
			RemoveSlowMo();
	}
	
	event OnEffectRemoved()
	{
		RemoveSlowMo();
		super.OnEffectRemoved();
	}

	private function InitiateSlowMo()
	{
		theGame.SetTimeScale(slowdownFactor, theGame.GetTimescaleSource(ETS_PotionBlizzard), theGame.GetTimescalePriority(ETS_PotionBlizzard));
		speedMultiplierID = target.SetAnimationSpeedMultiplier(1 / slowdownFactor);
	}
	
	private final function RemoveSlowMo()
	{
		target.ResetAnimationSpeedMultiplier(speedMultiplierID);
		theGame.RemoveTimeScale(theGame.GetTimescaleSource(ETS_PotionBlizzard));
		speedMultiplierID = -1;
	}
}